#ifndef vtk_libproj_mangle_h
#define vtk_libproj_mangle_h

#define aacos vtklibproj_aacos
#define aasin vtklibproj_aasin
#define aatan2 vtklibproj_aatan2
#define adjlon vtklibproj_adjlon
#define asqrt vtklibproj_asqrt
#define auth_lat vtklibproj_auth_lat
#define bch2bps vtklibproj_bch2bps
#define bcheval vtklibproj_bcheval
#define bchgen vtklibproj_bchgen
#define biveval vtklibproj_biveval
#define bpseval vtklibproj_bpseval
#define dmstor vtklibproj_dmstor
#define dmstor_ctx vtklibproj_dmstor_ctx
#define emess vtklibproj_emess
#define freev2 vtklibproj_freev2
#define geod_direct vtklibproj_geod_direct
#define geod_directline vtklibproj_geod_directline
#define geod_gendirect vtklibproj_geod_gendirect
#define geod_gendirectline vtklibproj_geod_gendirectline
#define geod_geninverse vtklibproj_geod_geninverse
#define geod_genposition vtklibproj_geod_genposition
#define geod_gensetdistance vtklibproj_geod_gensetdistance
#define geod_init vtklibproj_geod_init
#define geod_inverse vtklibproj_geod_inverse
#define geod_inverseline vtklibproj_geod_inverseline
#define geod_lineinit vtklibproj_geod_lineinit
#define geod_polygon_addedge vtklibproj_geod_polygon_addedge
#define geod_polygon_addpoint vtklibproj_geod_polygon_addpoint
#define geod_polygonarea vtklibproj_geod_polygonarea
#define geod_polygon_clear vtklibproj_geod_polygon_clear
#define geod_polygon_compute vtklibproj_geod_polygon_compute
#define geod_polygon_init vtklibproj_geod_polygon_init
#define geod_polygon_testedge vtklibproj_geod_polygon_testedge
#define geod_polygon_testpoint vtklibproj_geod_polygon_testpoint
#define geod_position vtklibproj_geod_position
#define geod_setdistance vtklibproj_geod_setdistance
#define healpix_sphere vtklibproj_healpix_sphere
#define healpix_sphere_inverse vtklibproj_healpix_sphere_inverse
#define in_image vtklibproj_in_image
#define mk_cheby vtklibproj_mk_cheby
#define nad_ctable2_init vtklibproj_nad_ctable2_init
#define nad_ctable2_load vtklibproj_nad_ctable2_load
#define nad_ctable_init vtklibproj_nad_ctable_init
#define nad_ctable_load vtklibproj_nad_ctable_load
#define nad_cvt vtklibproj_nad_cvt
#define nad_free vtklibproj_nad_free
#define nad_init vtklibproj_nad_init
#define nad_intr vtklibproj_nad_intr
#define pj_acquire_lock vtklibproj_pj_acquire_lock
#define pj_aea vtklibproj_pj_aea
#define pj_aea_selftest vtklibproj_pj_aea_selftest
#define pj_aeqd vtklibproj_pj_aeqd
#define pj_aeqd_selftest vtklibproj_pj_aeqd_selftest
#define pj_airy vtklibproj_pj_airy
#define pj_airy_selftest vtklibproj_pj_airy_selftest
#define pj_aitoff vtklibproj_pj_aitoff
#define pj_aitoff_selftest vtklibproj_pj_aitoff_selftest
#define pj_alsk vtklibproj_pj_alsk
#define pj_alsk_selftest vtklibproj_pj_alsk_selftest
#define pj_apian vtklibproj_pj_apian
#define pj_apian_selftest vtklibproj_pj_apian_selftest
#define pj_apply_gridshift vtklibproj_pj_apply_gridshift
#define pj_apply_gridshift_2 vtklibproj_pj_apply_gridshift_2
#define pj_apply_gridshift_3 vtklibproj_pj_apply_gridshift_3
#define pj_apply_vgridshift vtklibproj_pj_apply_vgridshift
#define pj_atof vtklibproj_pj_atof
#define pj_august vtklibproj_pj_august
#define pj_august_selftest vtklibproj_pj_august_selftest
#define pj_authlat vtklibproj_pj_authlat
#define pj_authset vtklibproj_pj_authset
#define pj_bacon vtklibproj_pj_bacon
#define pj_bacon_selftest vtklibproj_pj_bacon_selftest
#define pj_bipc vtklibproj_pj_bipc
#define pj_bipc_selftest vtklibproj_pj_bipc_selftest
#define pj_boggs vtklibproj_pj_boggs
#define pj_boggs_selftest vtklibproj_pj_boggs_selftest
#define pj_bonne vtklibproj_pj_bonne
#define pj_bonne_selftest vtklibproj_pj_bonne_selftest
#define pj_calcofi vtklibproj_pj_calcofi
#define pj_calcofi_selftest vtklibproj_pj_calcofi_selftest
#define pj_calloc vtklibproj_pj_calloc
#define pj_cass vtklibproj_pj_cass
#define pj_cass_selftest vtklibproj_pj_cass_selftest
#define pj_cc vtklibproj_pj_cc
#define pj_cc_selftest vtklibproj_pj_cc_selftest
#define pj_cea vtklibproj_pj_cea
#define pj_cea_selftest vtklibproj_pj_cea_selftest
#define pj_chamb vtklibproj_pj_chamb
#define pj_chamb_selftest vtklibproj_pj_chamb_selftest
#define pj_cleanup_lock vtklibproj_pj_cleanup_lock
#define pj_clear_initcache vtklibproj_pj_clear_initcache
#define pj_clone_paralist vtklibproj_pj_clone_paralist
#define pj_collg vtklibproj_pj_collg
#define pj_collg_selftest vtklibproj_pj_collg_selftest
#define pj_comill vtklibproj_pj_comill
#define pj_comill_selftest vtklibproj_pj_comill_selftest
#define pj_compare_datums vtklibproj_pj_compare_datums
#define pj_Convert_Geocentric_To_Geodetic vtklibproj_pj_Convert_Geocentric_To_Geodetic
#define pj_Convert_Geodetic_To_Geocentric vtklibproj_pj_Convert_Geodetic_To_Geocentric
#define pj_crast vtklibproj_pj_crast
#define pj_crast_selftest vtklibproj_pj_crast_selftest
#define pj_ctx_alloc vtklibproj_pj_ctx_alloc
#define pj_ctx_fclose vtklibproj_pj_ctx_fclose
#define pj_ctx_fgets vtklibproj_pj_ctx_fgets
#define pj_ctx_fopen vtklibproj_pj_ctx_fopen
#define pj_ctx_fread vtklibproj_pj_ctx_fread
#define pj_ctx_free vtklibproj_pj_ctx_free
#define pj_ctx_fseek vtklibproj_pj_ctx_fseek
#define pj_ctx_ftell vtklibproj_pj_ctx_ftell
#define pj_ctx_get_app_data vtklibproj_pj_ctx_get_app_data
#define pj_ctx_get_errno vtklibproj_pj_ctx_get_errno
#define pj_ctx_get_fileapi vtklibproj_pj_ctx_get_fileapi
#define pj_ctx_set_app_data vtklibproj_pj_ctx_set_app_data
#define pj_ctx_set_debug vtklibproj_pj_ctx_set_debug
#define pj_ctx_set_errno vtklibproj_pj_ctx_set_errno
#define pj_ctx_set_fileapi vtklibproj_pj_ctx_set_fileapi
#define pj_ctx_set_logger vtklibproj_pj_ctx_set_logger
#define pj_dalloc vtklibproj_pj_dalloc
#define pj_datums vtklibproj_pj_datums
#define pj_datum_set vtklibproj_pj_datum_set
#define pj_datum_transform vtklibproj_pj_datum_transform
#define pj_dealloc vtklibproj_pj_dealloc
#define pj_deallocate_grids vtklibproj_pj_deallocate_grids
#define pj_denoy vtklibproj_pj_denoy
#define pj_denoy_selftest vtklibproj_pj_denoy_selftest
#define pj_deriv vtklibproj_pj_deriv
#define pj_eck1 vtklibproj_pj_eck1
#define pj_eck1_selftest vtklibproj_pj_eck1_selftest
#define pj_eck2 vtklibproj_pj_eck2
#define pj_eck2_selftest vtklibproj_pj_eck2_selftest
#define pj_eck3 vtklibproj_pj_eck3
#define pj_eck3_selftest vtklibproj_pj_eck3_selftest
#define pj_eck4 vtklibproj_pj_eck4
#define pj_eck4_selftest vtklibproj_pj_eck4_selftest
#define pj_eck5 vtklibproj_pj_eck5
#define pj_eck5_selftest vtklibproj_pj_eck5_selftest
#define pj_eck6 vtklibproj_pj_eck6
#define pj_eck6_selftest vtklibproj_pj_eck6_selftest
#define pj_ellps vtklibproj_pj_ellps
#define pj_ell_set vtklibproj_pj_ell_set
#define pj_enfn vtklibproj_pj_enfn
#define pj_eqc vtklibproj_pj_eqc
#define pj_eqc_selftest vtklibproj_pj_eqc_selftest
#define pj_eqdc vtklibproj_pj_eqdc
#define pj_eqdc_selftest vtklibproj_pj_eqdc_selftest
#define pj_etmerc vtklibproj_pj_etmerc
#define pj_etmerc_selftest vtklibproj_pj_etmerc_selftest
#define pj_euler vtklibproj_pj_euler
#define pj_euler_selftest vtklibproj_pj_euler_selftest
#define pj_factors vtklibproj_pj_factors
#define pj_fahey vtklibproj_pj_fahey
#define pj_fahey_selftest vtklibproj_pj_fahey_selftest
#define pj_fouc vtklibproj_pj_fouc
#define pj_fouc_s vtklibproj_pj_fouc_s
#define pj_fouc_selftest vtklibproj_pj_fouc_selftest
#define pj_fouc_s_selftest vtklibproj_pj_fouc_s_selftest
#define pj_free vtklibproj_pj_free
#define pj_fwd vtklibproj_pj_fwd
#define pj_fwd3d vtklibproj_pj_fwd3d
#define pj_gall vtklibproj_pj_gall
#define pj_gall_selftest vtklibproj_pj_gall_selftest
#define pj_gauss vtklibproj_pj_gauss
#define pj_gauss_ini vtklibproj_pj_gauss_ini
#define pj_gc_apply_gridshift vtklibproj_pj_gc_apply_gridshift
#define pj_gc_findcatalog vtklibproj_pj_gc_findcatalog
#define pj_gc_findgrid vtklibproj_pj_gc_findgrid
#define pj_gc_parsedate vtklibproj_pj_gc_parsedate
#define pj_gc_readcatalog vtklibproj_pj_gc_readcatalog
#define pj_gc_unloadall vtklibproj_pj_gc_unloadall
#define pj_generic_selftest vtklibproj_pj_generic_selftest
#define pj_geocent vtklibproj_pj_geocent
#define pj_geocentric_from_wgs84 vtklibproj_pj_geocentric_from_wgs84
#define pj_geocentric_to_geodetic vtklibproj_pj_geocentric_to_geodetic
#define pj_geocentric_to_wgs84 vtklibproj_pj_geocentric_to_wgs84
#define pj_geocent_selftest vtklibproj_pj_geocent_selftest
#define pj_geodetic_to_geocentric vtklibproj_pj_geodetic_to_geocentric
#define pj_geos vtklibproj_pj_geos
#define pj_geos_selftest vtklibproj_pj_geos_selftest
#define pj_get_ctx vtklibproj_pj_get_ctx
#define pj_get_datums_ref vtklibproj_pj_get_datums_ref
#define pj_get_def vtklibproj_pj_get_def
#define pj_get_default_ctx vtklibproj_pj_get_default_ctx
#define pj_get_default_fileapi vtklibproj_pj_get_default_fileapi
#define pj_get_ellps_ref vtklibproj_pj_get_ellps_ref
#define pj_get_errno_ref vtklibproj_pj_get_errno_ref
#define pj_Get_Geocentric_Parameters vtklibproj_pj_Get_Geocentric_Parameters
#define pj_get_list_ref vtklibproj_pj_get_list_ref
#define pj_get_prime_meridians_ref vtklibproj_pj_get_prime_meridians_ref
#define pj_get_release vtklibproj_pj_get_release
#define pj_get_selftest_list_ref vtklibproj_pj_get_selftest_list_ref
#define pj_get_spheroid_defn vtklibproj_pj_get_spheroid_defn
#define pj_get_units_ref vtklibproj_pj_get_units_ref
#define pj_gins8 vtklibproj_pj_gins8
#define pj_gins8_selftest vtklibproj_pj_gins8_selftest
#define pj_gnom vtklibproj_pj_gnom
#define pj_gnom_selftest vtklibproj_pj_gnom_selftest
#define pj_gn_sinu vtklibproj_pj_gn_sinu
#define pj_gn_sinu_selftest vtklibproj_pj_gn_sinu_selftest
#define pj_goode vtklibproj_pj_goode
#define pj_goode_selftest vtklibproj_pj_goode_selftest
#define pj_gridinfo_free vtklibproj_pj_gridinfo_free
#define pj_gridinfo_init vtklibproj_pj_gridinfo_init
#define pj_gridinfo_load vtklibproj_pj_gridinfo_load
#define pj_gridlist_from_nadgrids vtklibproj_pj_gridlist_from_nadgrids
#define pj_gs48 vtklibproj_pj_gs48
#define pj_gs48_selftest vtklibproj_pj_gs48_selftest
#define pj_gs50 vtklibproj_pj_gs50
#define pj_gs50_selftest vtklibproj_pj_gs50_selftest
#define pj_gstmerc vtklibproj_pj_gstmerc
#define pj_gstmerc_selftest vtklibproj_pj_gstmerc_selftest
#define pj_hammer vtklibproj_pj_hammer
#define pj_hammer_selftest vtklibproj_pj_hammer_selftest
#define pj_hatano vtklibproj_pj_hatano
#define pj_hatano_selftest vtklibproj_pj_hatano_selftest
#define pj_healpix vtklibproj_pj_healpix
#define pj_healpix_selftest vtklibproj_pj_healpix_selftest
#define pj_igh vtklibproj_pj_igh
#define pj_igh_selftest vtklibproj_pj_igh_selftest
#define pj_imw_p vtklibproj_pj_imw_p
#define pj_imw_p_selftest vtklibproj_pj_imw_p_selftest
#define pj_init vtklibproj_pj_init
#define pj_init_ctx vtklibproj_pj_init_ctx
#define pj_init_plus vtklibproj_pj_init_plus
#define pj_init_plus_ctx vtklibproj_pj_init_plus_ctx
#define pj_insert_initcache vtklibproj_pj_insert_initcache
#define pj_inv vtklibproj_pj_inv
#define pj_inv3d vtklibproj_pj_inv3d
#define pj_inv_gauss vtklibproj_pj_inv_gauss
#define pj_inv_mlfn vtklibproj_pj_inv_mlfn
#define pj_isea vtklibproj_pj_isea
#define pj_isea_selftest vtklibproj_pj_isea_selftest
#define pj_is_geocent vtklibproj_pj_is_geocent
#define pj_is_latlong vtklibproj_pj_is_latlong
#define pj_kav5 vtklibproj_pj_kav5
#define pj_kav5_selftest vtklibproj_pj_kav5_selftest
#define pj_kav7 vtklibproj_pj_kav7
#define pj_kav7_selftest vtklibproj_pj_kav7_selftest
#define pj_krovak vtklibproj_pj_krovak
#define pj_krovak_selftest vtklibproj_pj_krovak_selftest
#define pj_labrd vtklibproj_pj_labrd
#define pj_labrd_selftest vtklibproj_pj_labrd_selftest
#define pj_laea vtklibproj_pj_laea
#define pj_laea_selftest vtklibproj_pj_laea_selftest
#define pj_lagrng vtklibproj_pj_lagrng
#define pj_lagrng_selftest vtklibproj_pj_lagrng_selftest
#define pj_larr vtklibproj_pj_larr
#define pj_larr_selftest vtklibproj_pj_larr_selftest
#define pj_lask vtklibproj_pj_lask
#define pj_lask_selftest vtklibproj_pj_lask_selftest
#define pj_latlon vtklibproj_pj_latlon
#define pj_latlong vtklibproj_pj_latlong
#define pj_latlong_from_proj vtklibproj_pj_latlong_from_proj
#define pj_latlong_selftest vtklibproj_pj_latlong_selftest
#define pj_latlon_selftest vtklibproj_pj_latlon_selftest
#define pj_lcc vtklibproj_pj_lcc
#define pj_lcca vtklibproj_pj_lcca
#define pj_lcca_selftest vtklibproj_pj_lcca_selftest
#define pj_lcc_selftest vtklibproj_pj_lcc_selftest
#define pj_leac vtklibproj_pj_leac
#define pj_leac_selftest vtklibproj_pj_leac_selftest
#define pj_lee_os vtklibproj_pj_lee_os
#define pj_lee_os_selftest vtklibproj_pj_lee_os_selftest
#define pj_list vtklibproj_pj_list
#define pj_log vtklibproj_pj_log
#define pj_longlat vtklibproj_pj_longlat
#define pj_longlat_selftest vtklibproj_pj_longlat_selftest
#define pj_lonlat vtklibproj_pj_lonlat
#define pj_lonlat_selftest vtklibproj_pj_lonlat_selftest
#define pj_loxim vtklibproj_pj_loxim
#define pj_loxim_selftest vtklibproj_pj_loxim_selftest
#define pj_lsat vtklibproj_pj_lsat
#define pj_lsat_selftest vtklibproj_pj_lsat_selftest
#define pj_malloc vtklibproj_pj_malloc
#define pj_mbtfpp vtklibproj_pj_mbtfpp
#define pj_mbtfpp_selftest vtklibproj_pj_mbtfpp_selftest
#define pj_mbtfpq vtklibproj_pj_mbtfpq
#define pj_mbtfpq_selftest vtklibproj_pj_mbtfpq_selftest
#define pj_mbtfps vtklibproj_pj_mbtfps
#define pj_mbt_fps vtklibproj_pj_mbt_fps
#define pj_mbt_fps_selftest vtklibproj_pj_mbt_fps_selftest
#define pj_mbtfps_selftest vtklibproj_pj_mbtfps_selftest
#define pj_mbt_s vtklibproj_pj_mbt_s
#define pj_mbt_s_selftest vtklibproj_pj_mbt_s_selftest
#define pj_merc vtklibproj_pj_merc
#define pj_merc_selftest vtklibproj_pj_merc_selftest
#define pj_mill vtklibproj_pj_mill
#define pj_mill_selftest vtklibproj_pj_mill_selftest
#define pj_mil_os vtklibproj_pj_mil_os
#define pj_mil_os_selftest vtklibproj_pj_mil_os_selftest
#define pj_misrsom vtklibproj_pj_misrsom
#define pj_misrsom_selftest vtklibproj_pj_misrsom_selftest
#define pj_mkparam vtklibproj_pj_mkparam
#define pj_mlfn vtklibproj_pj_mlfn
#define pj_moll vtklibproj_pj_moll
#define pj_moll_selftest vtklibproj_pj_moll_selftest
#define pj_msfn vtklibproj_pj_msfn
#define pj_murd1 vtklibproj_pj_murd1
#define pj_murd1_selftest vtklibproj_pj_murd1_selftest
#define pj_murd2 vtklibproj_pj_murd2
#define pj_murd2_selftest vtklibproj_pj_murd2_selftest
#define pj_murd3 vtklibproj_pj_murd3
#define pj_murd3_selftest vtklibproj_pj_murd3_selftest
#define pj_natearth vtklibproj_pj_natearth
#define pj_natearth2 vtklibproj_pj_natearth2
#define pj_natearth2_selftest vtklibproj_pj_natearth2_selftest
#define pj_natearth_selftest vtklibproj_pj_natearth_selftest
#define pj_nell vtklibproj_pj_nell
#define pj_nell_h vtklibproj_pj_nell_h
#define pj_nell_h_selftest vtklibproj_pj_nell_h_selftest
#define pj_nell_selftest vtklibproj_pj_nell_selftest
#define pj_nicol vtklibproj_pj_nicol
#define pj_nicol_selftest vtklibproj_pj_nicol_selftest
#define pj_nsper vtklibproj_pj_nsper
#define pj_nsper_selftest vtklibproj_pj_nsper_selftest
#define pj_nzmg vtklibproj_pj_nzmg
#define pj_nzmg_selftest vtklibproj_pj_nzmg_selftest
#define pj_ob_tran vtklibproj_pj_ob_tran
#define pj_ob_tran_selftest vtklibproj_pj_ob_tran_selftest
#define pj_ocea vtklibproj_pj_ocea
#define pj_ocea_selftest vtklibproj_pj_ocea_selftest
#define pj_oea vtklibproj_pj_oea
#define pj_oea_selftest vtklibproj_pj_oea_selftest
#define pj_omerc vtklibproj_pj_omerc
#define pj_omerc_selftest vtklibproj_pj_omerc_selftest
#define pj_open_lib vtklibproj_pj_open_lib
#define pj_ortel vtklibproj_pj_ortel
#define pj_ortel_selftest vtklibproj_pj_ortel_selftest
#define pj_ortho vtklibproj_pj_ortho
#define pj_ortho_selftest vtklibproj_pj_ortho_selftest
#define pj_param vtklibproj_pj_param
#define pj_patterson vtklibproj_pj_patterson
#define pj_patterson_selftest vtklibproj_pj_patterson_selftest
#define pj_pconic vtklibproj_pj_pconic
#define pj_pconic_selftest vtklibproj_pj_pconic_selftest
#define pj_phi2 vtklibproj_pj_phi2
#define pj_poly vtklibproj_pj_poly
#define pj_poly_selftest vtklibproj_pj_poly_selftest
#define pj_prepare vtklibproj_pj_prepare
#define pj_prime_meridians vtklibproj_pj_prime_meridians
#define pj_pr_list vtklibproj_pj_pr_list
#define pj_projection_specific_setup_aea vtklibproj_pj_projection_specific_setup_aea
#define pj_projection_specific_setup_aeqd vtklibproj_pj_projection_specific_setup_aeqd
#define pj_projection_specific_setup_airy vtklibproj_pj_projection_specific_setup_airy
#define pj_projection_specific_setup_aitoff vtklibproj_pj_projection_specific_setup_aitoff
#define pj_projection_specific_setup_alsk vtklibproj_pj_projection_specific_setup_alsk
#define pj_projection_specific_setup_apian vtklibproj_pj_projection_specific_setup_apian
#define pj_projection_specific_setup_august vtklibproj_pj_projection_specific_setup_august
#define pj_projection_specific_setup_bacon vtklibproj_pj_projection_specific_setup_bacon
#define pj_projection_specific_setup_bipc vtklibproj_pj_projection_specific_setup_bipc
#define pj_projection_specific_setup_boggs vtklibproj_pj_projection_specific_setup_boggs
#define pj_projection_specific_setup_bonne vtklibproj_pj_projection_specific_setup_bonne
#define pj_projection_specific_setup_calcofi vtklibproj_pj_projection_specific_setup_calcofi
#define pj_projection_specific_setup_cass vtklibproj_pj_projection_specific_setup_cass
#define pj_projection_specific_setup_cc vtklibproj_pj_projection_specific_setup_cc
#define pj_projection_specific_setup_cea vtklibproj_pj_projection_specific_setup_cea
#define pj_projection_specific_setup_chamb vtklibproj_pj_projection_specific_setup_chamb
#define pj_projection_specific_setup_collg vtklibproj_pj_projection_specific_setup_collg
#define pj_projection_specific_setup_comill vtklibproj_pj_projection_specific_setup_comill
#define pj_projection_specific_setup_crast vtklibproj_pj_projection_specific_setup_crast
#define pj_projection_specific_setup_denoy vtklibproj_pj_projection_specific_setup_denoy
#define pj_projection_specific_setup_eck1 vtklibproj_pj_projection_specific_setup_eck1
#define pj_projection_specific_setup_eck2 vtklibproj_pj_projection_specific_setup_eck2
#define pj_projection_specific_setup_eck3 vtklibproj_pj_projection_specific_setup_eck3
#define pj_projection_specific_setup_eck4 vtklibproj_pj_projection_specific_setup_eck4
#define pj_projection_specific_setup_eck5 vtklibproj_pj_projection_specific_setup_eck5
#define pj_projection_specific_setup_eck6 vtklibproj_pj_projection_specific_setup_eck6
#define pj_projection_specific_setup_eqc vtklibproj_pj_projection_specific_setup_eqc
#define pj_projection_specific_setup_eqdc vtklibproj_pj_projection_specific_setup_eqdc
#define pj_projection_specific_setup_etmerc vtklibproj_pj_projection_specific_setup_etmerc
#define pj_projection_specific_setup_euler vtklibproj_pj_projection_specific_setup_euler
#define pj_projection_specific_setup_fahey vtklibproj_pj_projection_specific_setup_fahey
#define pj_projection_specific_setup_fouc vtklibproj_pj_projection_specific_setup_fouc
#define pj_projection_specific_setup_fouc_s vtklibproj_pj_projection_specific_setup_fouc_s
#define pj_projection_specific_setup_gall vtklibproj_pj_projection_specific_setup_gall
#define pj_projection_specific_setup_geocent vtklibproj_pj_projection_specific_setup_geocent
#define pj_projection_specific_setup_geos vtklibproj_pj_projection_specific_setup_geos
#define pj_projection_specific_setup_gins8 vtklibproj_pj_projection_specific_setup_gins8
#define pj_projection_specific_setup_gnom vtklibproj_pj_projection_specific_setup_gnom
#define pj_projection_specific_setup_gn_sinu vtklibproj_pj_projection_specific_setup_gn_sinu
#define pj_projection_specific_setup_goode vtklibproj_pj_projection_specific_setup_goode
#define pj_projection_specific_setup_gs48 vtklibproj_pj_projection_specific_setup_gs48
#define pj_projection_specific_setup_gs50 vtklibproj_pj_projection_specific_setup_gs50
#define pj_projection_specific_setup_gstmerc vtklibproj_pj_projection_specific_setup_gstmerc
#define pj_projection_specific_setup_hammer vtklibproj_pj_projection_specific_setup_hammer
#define pj_projection_specific_setup_hatano vtklibproj_pj_projection_specific_setup_hatano
#define pj_projection_specific_setup_healpix vtklibproj_pj_projection_specific_setup_healpix
#define pj_projection_specific_setup_igh vtklibproj_pj_projection_specific_setup_igh
#define pj_projection_specific_setup_imw_p vtklibproj_pj_projection_specific_setup_imw_p
#define pj_projection_specific_setup_isea vtklibproj_pj_projection_specific_setup_isea
#define pj_projection_specific_setup_kav5 vtklibproj_pj_projection_specific_setup_kav5
#define pj_projection_specific_setup_kav7 vtklibproj_pj_projection_specific_setup_kav7
#define pj_projection_specific_setup_krovak vtklibproj_pj_projection_specific_setup_krovak
#define pj_projection_specific_setup_labrd vtklibproj_pj_projection_specific_setup_labrd
#define pj_projection_specific_setup_laea vtklibproj_pj_projection_specific_setup_laea
#define pj_projection_specific_setup_lagrng vtklibproj_pj_projection_specific_setup_lagrng
#define pj_projection_specific_setup_larr vtklibproj_pj_projection_specific_setup_larr
#define pj_projection_specific_setup_lask vtklibproj_pj_projection_specific_setup_lask
#define pj_projection_specific_setup_latlon vtklibproj_pj_projection_specific_setup_latlon
#define pj_projection_specific_setup_latlong vtklibproj_pj_projection_specific_setup_latlong
#define pj_projection_specific_setup_lcc vtklibproj_pj_projection_specific_setup_lcc
#define pj_projection_specific_setup_lcca vtklibproj_pj_projection_specific_setup_lcca
#define pj_projection_specific_setup_leac vtklibproj_pj_projection_specific_setup_leac
#define pj_projection_specific_setup_lee_os vtklibproj_pj_projection_specific_setup_lee_os
#define pj_projection_specific_setup_longlat vtklibproj_pj_projection_specific_setup_longlat
#define pj_projection_specific_setup_lonlat vtklibproj_pj_projection_specific_setup_lonlat
#define pj_projection_specific_setup_loxim vtklibproj_pj_projection_specific_setup_loxim
#define pj_projection_specific_setup_lsat vtklibproj_pj_projection_specific_setup_lsat
#define pj_projection_specific_setup_mbtfpp vtklibproj_pj_projection_specific_setup_mbtfpp
#define pj_projection_specific_setup_mbtfpq vtklibproj_pj_projection_specific_setup_mbtfpq
#define pj_projection_specific_setup_mbtfps vtklibproj_pj_projection_specific_setup_mbtfps
#define pj_projection_specific_setup_mbt_fps vtklibproj_pj_projection_specific_setup_mbt_fps
#define pj_projection_specific_setup_mbt_s vtklibproj_pj_projection_specific_setup_mbt_s
#define pj_projection_specific_setup_merc vtklibproj_pj_projection_specific_setup_merc
#define pj_projection_specific_setup_mill vtklibproj_pj_projection_specific_setup_mill
#define pj_projection_specific_setup_mil_os vtklibproj_pj_projection_specific_setup_mil_os
#define pj_projection_specific_setup_misrsom vtklibproj_pj_projection_specific_setup_misrsom
#define pj_projection_specific_setup_moll vtklibproj_pj_projection_specific_setup_moll
#define pj_projection_specific_setup_murd1 vtklibproj_pj_projection_specific_setup_murd1
#define pj_projection_specific_setup_murd2 vtklibproj_pj_projection_specific_setup_murd2
#define pj_projection_specific_setup_murd3 vtklibproj_pj_projection_specific_setup_murd3
#define pj_projection_specific_setup_natearth vtklibproj_pj_projection_specific_setup_natearth
#define pj_projection_specific_setup_natearth2 vtklibproj_pj_projection_specific_setup_natearth2
#define pj_projection_specific_setup_nell vtklibproj_pj_projection_specific_setup_nell
#define pj_projection_specific_setup_nell_h vtklibproj_pj_projection_specific_setup_nell_h
#define pj_projection_specific_setup_nicol vtklibproj_pj_projection_specific_setup_nicol
#define pj_projection_specific_setup_nsper vtklibproj_pj_projection_specific_setup_nsper
#define pj_projection_specific_setup_nzmg vtklibproj_pj_projection_specific_setup_nzmg
#define pj_projection_specific_setup_ob_tran vtklibproj_pj_projection_specific_setup_ob_tran
#define pj_projection_specific_setup_ocea vtklibproj_pj_projection_specific_setup_ocea
#define pj_projection_specific_setup_oea vtklibproj_pj_projection_specific_setup_oea
#define pj_projection_specific_setup_omerc vtklibproj_pj_projection_specific_setup_omerc
#define pj_projection_specific_setup_ortel vtklibproj_pj_projection_specific_setup_ortel
#define pj_projection_specific_setup_ortho vtklibproj_pj_projection_specific_setup_ortho
#define pj_projection_specific_setup_patterson vtklibproj_pj_projection_specific_setup_patterson
#define pj_projection_specific_setup_pconic vtklibproj_pj_projection_specific_setup_pconic
#define pj_projection_specific_setup_poly vtklibproj_pj_projection_specific_setup_poly
#define pj_projection_specific_setup_putp1 vtklibproj_pj_projection_specific_setup_putp1
#define pj_projection_specific_setup_putp2 vtklibproj_pj_projection_specific_setup_putp2
#define pj_projection_specific_setup_putp3 vtklibproj_pj_projection_specific_setup_putp3
#define pj_projection_specific_setup_putp3p vtklibproj_pj_projection_specific_setup_putp3p
#define pj_projection_specific_setup_putp4p vtklibproj_pj_projection_specific_setup_putp4p
#define pj_projection_specific_setup_putp5 vtklibproj_pj_projection_specific_setup_putp5
#define pj_projection_specific_setup_putp5p vtklibproj_pj_projection_specific_setup_putp5p
#define pj_projection_specific_setup_putp6 vtklibproj_pj_projection_specific_setup_putp6
#define pj_projection_specific_setup_putp6p vtklibproj_pj_projection_specific_setup_putp6p
#define pj_projection_specific_setup_qsc vtklibproj_pj_projection_specific_setup_qsc
#define pj_projection_specific_setup_qua_aut vtklibproj_pj_projection_specific_setup_qua_aut
#define pj_projection_specific_setup_rhealpix vtklibproj_pj_projection_specific_setup_rhealpix
#define pj_projection_specific_setup_robin vtklibproj_pj_projection_specific_setup_robin
#define pj_projection_specific_setup_rouss vtklibproj_pj_projection_specific_setup_rouss
#define pj_projection_specific_setup_rpoly vtklibproj_pj_projection_specific_setup_rpoly
#define pj_projection_specific_setup_sch vtklibproj_pj_projection_specific_setup_sch
#define pj_projection_specific_setup_sinu vtklibproj_pj_projection_specific_setup_sinu
#define pj_projection_specific_setup_somerc vtklibproj_pj_projection_specific_setup_somerc
#define pj_projection_specific_setup_stere vtklibproj_pj_projection_specific_setup_stere
#define pj_projection_specific_setup_sterea vtklibproj_pj_projection_specific_setup_sterea
#define pj_projection_specific_setup_tcc vtklibproj_pj_projection_specific_setup_tcc
#define pj_projection_specific_setup_tcea vtklibproj_pj_projection_specific_setup_tcea
#define pj_projection_specific_setup_times vtklibproj_pj_projection_specific_setup_times
#define pj_projection_specific_setup_tissot vtklibproj_pj_projection_specific_setup_tissot
#define pj_projection_specific_setup_tmerc vtklibproj_pj_projection_specific_setup_tmerc
#define pj_projection_specific_setup_tpeqd vtklibproj_pj_projection_specific_setup_tpeqd
#define pj_projection_specific_setup_tpers vtklibproj_pj_projection_specific_setup_tpers
#define pj_projection_specific_setup_ups vtklibproj_pj_projection_specific_setup_ups
#define pj_projection_specific_setup_urm5 vtklibproj_pj_projection_specific_setup_urm5
#define pj_projection_specific_setup_urmfps vtklibproj_pj_projection_specific_setup_urmfps
#define pj_projection_specific_setup_utm vtklibproj_pj_projection_specific_setup_utm
#define pj_projection_specific_setup_vandg vtklibproj_pj_projection_specific_setup_vandg
#define pj_projection_specific_setup_vandg2 vtklibproj_pj_projection_specific_setup_vandg2
#define pj_projection_specific_setup_vandg3 vtklibproj_pj_projection_specific_setup_vandg3
#define pj_projection_specific_setup_vandg4 vtklibproj_pj_projection_specific_setup_vandg4
#define pj_projection_specific_setup_vitk1 vtklibproj_pj_projection_specific_setup_vitk1
#define pj_projection_specific_setup_wag1 vtklibproj_pj_projection_specific_setup_wag1
#define pj_projection_specific_setup_wag2 vtklibproj_pj_projection_specific_setup_wag2
#define pj_projection_specific_setup_wag3 vtklibproj_pj_projection_specific_setup_wag3
#define pj_projection_specific_setup_wag4 vtklibproj_pj_projection_specific_setup_wag4
#define pj_projection_specific_setup_wag5 vtklibproj_pj_projection_specific_setup_wag5
#define pj_projection_specific_setup_wag6 vtklibproj_pj_projection_specific_setup_wag6
#define pj_projection_specific_setup_wag7 vtklibproj_pj_projection_specific_setup_wag7
#define pj_projection_specific_setup_weren vtklibproj_pj_projection_specific_setup_weren
#define pj_projection_specific_setup_wink1 vtklibproj_pj_projection_specific_setup_wink1
#define pj_projection_specific_setup_wink2 vtklibproj_pj_projection_specific_setup_wink2
#define pj_projection_specific_setup_wintri vtklibproj_pj_projection_specific_setup_wintri
#define pj_putp1 vtklibproj_pj_putp1
#define pj_putp1_selftest vtklibproj_pj_putp1_selftest
#define pj_putp2 vtklibproj_pj_putp2
#define pj_putp2_selftest vtklibproj_pj_putp2_selftest
#define pj_putp3 vtklibproj_pj_putp3
#define pj_putp3p vtklibproj_pj_putp3p
#define pj_putp3p_selftest vtklibproj_pj_putp3p_selftest
#define pj_putp3_selftest vtklibproj_pj_putp3_selftest
#define pj_putp4p vtklibproj_pj_putp4p
#define pj_putp4p_selftest vtklibproj_pj_putp4p_selftest
#define pj_putp5 vtklibproj_pj_putp5
#define pj_putp5p vtklibproj_pj_putp5p
#define pj_putp5p_selftest vtklibproj_pj_putp5p_selftest
#define pj_putp5_selftest vtklibproj_pj_putp5_selftest
#define pj_putp6 vtklibproj_pj_putp6
#define pj_putp6p vtklibproj_pj_putp6p
#define pj_putp6p_selftest vtklibproj_pj_putp6p_selftest
#define pj_putp6_selftest vtklibproj_pj_putp6_selftest
#define pj_qsc vtklibproj_pj_qsc
#define pj_qsc_selftest vtklibproj_pj_qsc_selftest
#define pj_qsfn vtklibproj_pj_qsfn
#define pj_qua_aut vtklibproj_pj_qua_aut
#define pj_qua_aut_selftest vtklibproj_pj_qua_aut_selftest
#define pj_release vtklibproj_pj_release
#define pj_release_lock vtklibproj_pj_release_lock
#define pj_rhealpix vtklibproj_pj_rhealpix
#define pj_rhealpix_selftest vtklibproj_pj_rhealpix_selftest
#define pj_robin vtklibproj_pj_robin
#define pj_robin_selftest vtklibproj_pj_robin_selftest
#define pj_rouss vtklibproj_pj_rouss
#define pj_rouss_selftest vtklibproj_pj_rouss_selftest
#define pj_rpoly vtklibproj_pj_rpoly
#define pj_rpoly_selftest vtklibproj_pj_rpoly_selftest
#define pj_run_selftests vtklibproj_pj_run_selftests
#define pj_s_aea vtklibproj_pj_s_aea
#define pj_s_aeqd vtklibproj_pj_s_aeqd
#define pj_s_airy vtklibproj_pj_s_airy
#define pj_s_aitoff vtklibproj_pj_s_aitoff
#define pj_s_alsk vtklibproj_pj_s_alsk
#define pj_s_apian vtklibproj_pj_s_apian
#define pj_s_august vtklibproj_pj_s_august
#define pj_s_bacon vtklibproj_pj_s_bacon
#define pj_s_bipc vtklibproj_pj_s_bipc
#define pj_s_boggs vtklibproj_pj_s_boggs
#define pj_s_bonne vtklibproj_pj_s_bonne
#define pj_s_calcofi vtklibproj_pj_s_calcofi
#define pj_s_cass vtklibproj_pj_s_cass
#define pj_s_cc vtklibproj_pj_s_cc
#define pj_s_cea vtklibproj_pj_s_cea
#define pj_sch vtklibproj_pj_sch
#define pj_s_chamb vtklibproj_pj_s_chamb
#define pj_sch_selftest vtklibproj_pj_sch_selftest
#define pj_s_collg vtklibproj_pj_s_collg
#define pj_s_comill vtklibproj_pj_s_comill
#define pj_s_crast vtklibproj_pj_s_crast
#define pj_s_denoy vtklibproj_pj_s_denoy
#define pj_search_initcache vtklibproj_pj_search_initcache
#define pj_s_eck1 vtklibproj_pj_s_eck1
#define pj_s_eck2 vtklibproj_pj_s_eck2
#define pj_s_eck3 vtklibproj_pj_s_eck3
#define pj_s_eck4 vtklibproj_pj_s_eck4
#define pj_s_eck5 vtklibproj_pj_s_eck5
#define pj_s_eck6 vtklibproj_pj_s_eck6
#define pj_selftest_list vtklibproj_pj_selftest_list
#define pj_s_eqc vtklibproj_pj_s_eqc
#define pj_s_eqdc vtklibproj_pj_s_eqdc
#define pj_set_ctx vtklibproj_pj_set_ctx
#define pj_set_finder vtklibproj_pj_set_finder
#define pj_Set_Geocentric_Parameters vtklibproj_pj_Set_Geocentric_Parameters
#define pj_s_etmerc vtklibproj_pj_s_etmerc
#define pj_set_searchpath vtklibproj_pj_set_searchpath
#define pj_s_euler vtklibproj_pj_s_euler
#define pj_s_fahey vtklibproj_pj_s_fahey
#define pj_s_fouc vtklibproj_pj_s_fouc
#define pj_s_fouc_s vtklibproj_pj_s_fouc_s
#define pj_s_gall vtklibproj_pj_s_gall
#define pj_s_geocent vtklibproj_pj_s_geocent
#define pj_s_geos vtklibproj_pj_s_geos
#define pj_s_gins8 vtklibproj_pj_s_gins8
#define pj_s_gnom vtklibproj_pj_s_gnom
#define pj_s_gn_sinu vtklibproj_pj_s_gn_sinu
#define pj_s_goode vtklibproj_pj_s_goode
#define pj_s_gs48 vtklibproj_pj_s_gs48
#define pj_s_gs50 vtklibproj_pj_s_gs50
#define pj_s_gstmerc vtklibproj_pj_s_gstmerc
#define pj_s_hammer vtklibproj_pj_s_hammer
#define pj_s_hatano vtklibproj_pj_s_hatano
#define pj_s_healpix vtklibproj_pj_s_healpix
#define pj_s_igh vtklibproj_pj_s_igh
#define pj_sign vtklibproj_pj_sign
#define pj_s_imw_p vtklibproj_pj_s_imw_p
#define pj_sinu vtklibproj_pj_sinu
#define pj_sinu_selftest vtklibproj_pj_sinu_selftest
#define pj_s_isea vtklibproj_pj_s_isea
#define pj_s_kav5 vtklibproj_pj_s_kav5
#define pj_s_kav7 vtklibproj_pj_s_kav7
#define pj_s_krovak vtklibproj_pj_s_krovak
#define pj_s_labrd vtklibproj_pj_s_labrd
#define pj_s_laea vtklibproj_pj_s_laea
#define pj_s_lagrng vtklibproj_pj_s_lagrng
#define pj_s_larr vtklibproj_pj_s_larr
#define pj_s_lask vtklibproj_pj_s_lask
#define pj_s_latlon vtklibproj_pj_s_latlon
#define pj_s_latlong vtklibproj_pj_s_latlong
#define pj_s_lcc vtklibproj_pj_s_lcc
#define pj_s_lcca vtklibproj_pj_s_lcca
#define pj_s_leac vtklibproj_pj_s_leac
#define pj_s_lee_os vtklibproj_pj_s_lee_os
#define pj_s_longlat vtklibproj_pj_s_longlat
#define pj_s_lonlat vtklibproj_pj_s_lonlat
#define pj_s_loxim vtklibproj_pj_s_loxim
#define pj_s_lsat vtklibproj_pj_s_lsat
#define pj_s_mbtfpp vtklibproj_pj_s_mbtfpp
#define pj_s_mbtfpq vtklibproj_pj_s_mbtfpq
#define pj_s_mbtfps vtklibproj_pj_s_mbtfps
#define pj_s_mbt_fps vtklibproj_pj_s_mbt_fps
#define pj_s_mbt_s vtklibproj_pj_s_mbt_s
#define pj_s_merc vtklibproj_pj_s_merc
#define pj_s_mill vtklibproj_pj_s_mill
#define pj_s_mil_os vtklibproj_pj_s_mil_os
#define pj_s_misrsom vtklibproj_pj_s_misrsom
#define pj_s_moll vtklibproj_pj_s_moll
#define pj_s_murd1 vtklibproj_pj_s_murd1
#define pj_s_murd2 vtklibproj_pj_s_murd2
#define pj_s_murd3 vtklibproj_pj_s_murd3
#define pj_s_natearth vtklibproj_pj_s_natearth
#define pj_s_natearth2 vtklibproj_pj_s_natearth2
#define pj_s_nell vtklibproj_pj_s_nell
#define pj_s_nell_h vtklibproj_pj_s_nell_h
#define pj_s_nicol vtklibproj_pj_s_nicol
#define pj_s_nsper vtklibproj_pj_s_nsper
#define pj_s_nzmg vtklibproj_pj_s_nzmg
#define pj_s_ob_tran vtklibproj_pj_s_ob_tran
#define pj_s_ocea vtklibproj_pj_s_ocea
#define pj_s_oea vtklibproj_pj_s_oea
#define pj_somerc vtklibproj_pj_somerc
#define pj_s_omerc vtklibproj_pj_s_omerc
#define pj_somerc_selftest vtklibproj_pj_somerc_selftest
#define pj_s_ortel vtklibproj_pj_s_ortel
#define pj_s_ortho vtklibproj_pj_s_ortho
#define pj_s_patterson vtklibproj_pj_s_patterson
#define pj_s_pconic vtklibproj_pj_s_pconic
#define pj_s_poly vtklibproj_pj_s_poly
#define pj_s_putp1 vtklibproj_pj_s_putp1
#define pj_s_putp2 vtklibproj_pj_s_putp2
#define pj_s_putp3 vtklibproj_pj_s_putp3
#define pj_s_putp3p vtklibproj_pj_s_putp3p
#define pj_s_putp4p vtklibproj_pj_s_putp4p
#define pj_s_putp5 vtklibproj_pj_s_putp5
#define pj_s_putp5p vtklibproj_pj_s_putp5p
#define pj_s_putp6 vtklibproj_pj_s_putp6
#define pj_s_putp6p vtklibproj_pj_s_putp6p
#define pj_s_qsc vtklibproj_pj_s_qsc
#define pj_s_qua_aut vtklibproj_pj_s_qua_aut
#define pj_s_rhealpix vtklibproj_pj_s_rhealpix
#define pj_s_robin vtklibproj_pj_s_robin
#define pj_s_rouss vtklibproj_pj_s_rouss
#define pj_s_rpoly vtklibproj_pj_s_rpoly
#define pj_s_sch vtklibproj_pj_s_sch
#define pj_s_sinu vtklibproj_pj_s_sinu
#define pj_s_somerc vtklibproj_pj_s_somerc
#define pj_s_stere vtklibproj_pj_s_stere
#define pj_s_sterea vtklibproj_pj_s_sterea
#define pj_s_tcc vtklibproj_pj_s_tcc
#define pj_s_tcea vtklibproj_pj_s_tcea
#define pj_stderr_logger vtklibproj_pj_stderr_logger
#define pj_stere vtklibproj_pj_stere
#define pj_sterea vtklibproj_pj_sterea
#define pj_sterea_selftest vtklibproj_pj_sterea_selftest
#define pj_stere_selftest vtklibproj_pj_stere_selftest
#define pj_s_times vtklibproj_pj_s_times
#define pj_s_tissot vtklibproj_pj_s_tissot
#define pj_s_tmerc vtklibproj_pj_s_tmerc
#define pj_s_tpeqd vtklibproj_pj_s_tpeqd
#define pj_s_tpers vtklibproj_pj_s_tpers
#define pj_strerrno vtklibproj_pj_strerrno
#define pj_strtod vtklibproj_pj_strtod
#define pj_s_ups vtklibproj_pj_s_ups
#define pj_s_urm5 vtklibproj_pj_s_urm5
#define pj_s_urmfps vtklibproj_pj_s_urmfps
#define pj_s_utm vtklibproj_pj_s_utm
#define pj_s_vandg vtklibproj_pj_s_vandg
#define pj_s_vandg2 vtklibproj_pj_s_vandg2
#define pj_s_vandg3 vtklibproj_pj_s_vandg3
#define pj_s_vandg4 vtklibproj_pj_s_vandg4
#define pj_s_vitk1 vtklibproj_pj_s_vitk1
#define pj_s_wag1 vtklibproj_pj_s_wag1
#define pj_s_wag2 vtklibproj_pj_s_wag2
#define pj_s_wag3 vtklibproj_pj_s_wag3
#define pj_s_wag4 vtklibproj_pj_s_wag4
#define pj_s_wag5 vtklibproj_pj_s_wag5
#define pj_s_wag6 vtklibproj_pj_s_wag6
#define pj_s_wag7 vtklibproj_pj_s_wag7
#define pj_s_weren vtklibproj_pj_s_weren
#define pj_s_wink1 vtklibproj_pj_s_wink1
#define pj_s_wink2 vtklibproj_pj_s_wink2
#define pj_s_wintri vtklibproj_pj_s_wintri
#define pj_tcc vtklibproj_pj_tcc
#define pj_tcc_selftest vtklibproj_pj_tcc_selftest
#define pj_tcea vtklibproj_pj_tcea
#define pj_tcea_selftest vtklibproj_pj_tcea_selftest
#define pj_times vtklibproj_pj_times
#define pj_times_selftest vtklibproj_pj_times_selftest
#define pj_tissot vtklibproj_pj_tissot
#define pj_tissot_selftest vtklibproj_pj_tissot_selftest
#define pj_tmerc vtklibproj_pj_tmerc
#define pj_tmerc_selftest vtklibproj_pj_tmerc_selftest
#define pj_tpeqd vtklibproj_pj_tpeqd
#define pj_tpeqd_selftest vtklibproj_pj_tpeqd_selftest
#define pj_tpers vtklibproj_pj_tpers
#define pj_tpers_selftest vtklibproj_pj_tpers_selftest
#define pj_transform vtklibproj_pj_transform
#define pj_tsfn vtklibproj_pj_tsfn
#define pj_units vtklibproj_pj_units
#define pj_ups vtklibproj_pj_ups
#define pj_ups_selftest vtklibproj_pj_ups_selftest
#define pj_urm5 vtklibproj_pj_urm5
#define pj_urm5_selftest vtklibproj_pj_urm5_selftest
#define pj_urmfps vtklibproj_pj_urmfps
#define pj_urmfps_selftest vtklibproj_pj_urmfps_selftest
#define pj_utm vtklibproj_pj_utm
#define pj_utm_selftest vtklibproj_pj_utm_selftest
#define pj_vandg vtklibproj_pj_vandg
#define pj_vandg2 vtklibproj_pj_vandg2
#define pj_vandg2_selftest vtklibproj_pj_vandg2_selftest
#define pj_vandg3 vtklibproj_pj_vandg3
#define pj_vandg3_selftest vtklibproj_pj_vandg3_selftest
#define pj_vandg4 vtklibproj_pj_vandg4
#define pj_vandg4_selftest vtklibproj_pj_vandg4_selftest
#define pj_vandg_selftest vtklibproj_pj_vandg_selftest
#define pj_vitk1 vtklibproj_pj_vitk1
#define pj_vitk1_selftest vtklibproj_pj_vitk1_selftest
#define pj_wag1 vtklibproj_pj_wag1
#define pj_wag1_selftest vtklibproj_pj_wag1_selftest
#define pj_wag2 vtklibproj_pj_wag2
#define pj_wag2_selftest vtklibproj_pj_wag2_selftest
#define pj_wag3 vtklibproj_pj_wag3
#define pj_wag3_selftest vtklibproj_pj_wag3_selftest
#define pj_wag4 vtklibproj_pj_wag4
#define pj_wag4_selftest vtklibproj_pj_wag4_selftest
#define pj_wag5 vtklibproj_pj_wag5
#define pj_wag5_selftest vtklibproj_pj_wag5_selftest
#define pj_wag6 vtklibproj_pj_wag6
#define pj_wag6_selftest vtklibproj_pj_wag6_selftest
#define pj_wag7 vtklibproj_pj_wag7
#define pj_wag7_selftest vtklibproj_pj_wag7_selftest
#define pj_weren vtklibproj_pj_weren
#define pj_weren_selftest vtklibproj_pj_weren_selftest
#define pj_wink1 vtklibproj_pj_wink1
#define pj_wink1_selftest vtklibproj_pj_wink1_selftest
#define pj_wink2 vtklibproj_pj_wink2
#define pj_wink2_selftest vtklibproj_pj_wink2_selftest
#define pj_wintri vtklibproj_pj_wintri
#define pj_wintri_selftest vtklibproj_pj_wintri_selftest
#define pj_zpoly1 vtklibproj_pj_zpoly1
#define pj_zpolyd1 vtklibproj_pj_zpolyd1
#define proj_inv_mdist vtklibproj_proj_inv_mdist
#define proj_mdist vtklibproj_proj_mdist
#define proj_mdist_ini vtklibproj_proj_mdist_ini
#define rot vtklibproj_rot
#define rtodms vtklibproj_rtodms
#define set_rtodms vtklibproj_set_rtodms
#define vector1 vtklibproj_vector1
#define vector2 vtklibproj_vector2

#endif
