
#ifndef VTKDOUBLECONVERSION_EXPORT_H
#define VTKDOUBLECONVERSION_EXPORT_H

#ifdef VTKDOUBLECONVERSION_STATIC_DEFINE
#  define VTKDOUBLECONVERSION_EXPORT
#  define VTKDOUBLECONVERSION_NO_EXPORT
#else
#  ifndef VTKDOUBLECONVERSION_EXPORT
#    ifdef vtkdoubleconversion_EXPORTS
        /* We are building this library */
#      define VTKDOUBLECONVERSION_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKDOUBLECONVERSION_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKDOUBLECONVERSION_NO_EXPORT
#    define VTKDOUBLECONVERSION_NO_EXPORT 
#  endif
#endif

#ifndef VTKDOUBLECONVERSION_DEPRECATED
#  define VTKDOUBLECONVERSION_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKDOUBLECONVERSION_DEPRECATED_EXPORT
#  define VTKDOUBLECONVERSION_DEPRECATED_EXPORT VTKDOUBLECONVERSION_EXPORT VTKDOUBLECONVERSION_DEPRECATED
#endif

#ifndef VTKDOUBLECONVERSION_DEPRECATED_NO_EXPORT
#  define VTKDOUBLECONVERSION_DEPRECATED_NO_EXPORT VTKDOUBLECONVERSION_NO_EXPORT VTKDOUBLECONVERSION_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKDOUBLECONVERSION_NO_DEPRECATED
#    define VTKDOUBLECONVERSION_NO_DEPRECATED
#  endif
#endif

#endif /* VTKDOUBLECONVERSION_EXPORT_H */
